@@ -66,6 +66,9 @@ lavData <- function(data              = NULL,          # data.frame
     if(is.null(warn)) {
         warn <- TRUE
     }
+    if(allow.single.case) { # eg, in lavPredict
+        warn <- FALSE
+    }
 
     # four scenarios:
     #    0) data is already a lavData object: do nothing
@@ -192,9 +195,9 @@ lavData <- function(data              = NULL,          # data.frame
         }
 
         # get ov.names
-        if (is.null(ov.names)) {
+        if(is.null(ov.names)) {
             ov.names <- lapply(sample.cov, row.names)
-        } else if (!is.list(ov.names)) {
+        } else if(!is.list(ov.names)) {
             # duplicate ov.names for each group
             tmp <- ov.names; ov.names <- vector("list", length = ngroups)
             ov.names[1:ngroups] <- list(tmp)
@@ -242,6 +245,24 @@ lavData <- function(data              = NULL,          # data.frame
             warning("lavaan WARNING: std.ov argument is ignored if only sample statistics are provided.")
         }
 
+        # check variances (new in 0.6-7)
+        for(g in seq_len(ngroups)) {
+            VAR <- diag(sample.cov[[g]])
+            # 1. finite?
+            if(!all(is.finite(VAR))) {
+                stop("lavaan ERROR: at least one variance in the sample covariance matrix is not finite.")
+            }
+            # 2. near zero (or negative)?
+            if(any(VAR < .Machine$double.eps)) {
+                stop("lavaan ERROR: at least one variance in the sample covariance matrix is (near) zero or negative.")
+            }
+            # 3. very large?
+            max.var <- max(VAR)
+            if(max.var > 1000000) {
+                warning("lavaan WARNING: some observed variances in the sample covariance matrix are larger than 1000000.")
+            }
+        }
+
         # construct lavData object
         lavData <- new("lavData",
                        data.type   = "moment",
@@ -368,10 +389,12 @@ lavData <- function(data              = NULL,          # data.frame
         Lp <- vector("list", length = ngroups)
         for(g in 1:ngroups) {
             if(nlevels > 1L) {
+                # ALWAYS add ov.names.x at the end, even if conditional.x
+                OV.NAMES <- unique(c(ov.names[[g]], ov.names.x[[g]]))
                 Lp[[g]] <- lav_data_cluster_patterns(Y = NULL, clus = NULL,
                                                  cluster = cluster,
                                                  multilevel = TRUE,
-                                                 ov.names = ov.names[[g]],
+                                                 ov.names = OV.NAMES,
                                                  ov.names.l = ov.names.l[[g]])
             }
         } # g
@@ -637,9 +660,18 @@ lav_data_full <- function(data          = NULL,          # data.frame
         if(warn && any(f.names %in% unlist(ov.names.x)))
             warning(paste("lavaan WARNING: exogenous variable(s) declared as ordered in data:", paste(f.names, collapse=" ")))
     }
+    # check for ordered endogenous variables with more than 12 levels
+    if("ordered" %in% ov$type[!ov$name %in% unlist(ov.names.x)]) {
+        f.names <- ov$name[ov$type == "ordered" &
+                           !ov$name %in% unlist(ov.names.x) &
+                           ov$nlev > 12L]
+        if(warn && length(f.names) > 0L) {
+            warning(paste("lavaan WARNING: some ordered categorical variable(s) have more than 12 levels:", paste(f.names, collapse=" ")))
+        }
+    }
     # check for zero-cases
     idx <- which(ov$nobs == 0L | ov$var == 0)
-    if(length(idx) > 0L) {
+    if(!allow.single.case && length(idx) > 0L) {
         OV <- as.data.frame(ov)
         rn <- rownames(OV)
         rn[idx] <- paste(rn[idx], "***", sep="")
@@ -659,7 +691,7 @@ lav_data_full <- function(data          = NULL,          # data.frame
     }
     # check for ordered variables with only 1 level
     idx <- which(ov$type == "ordered" & ov$nlev == 1L)
-    if(length(idx) > 0L) {
+    if(!allow.single.case && length(idx) > 0L) {
         OV <- as.data.frame(ov)
         rn <- rownames(OV)
         rn[idx] <- paste(rn[idx], "***", sep="")
@@ -711,7 +743,13 @@ lav_data_full <- function(data          = NULL,          # data.frame
     for(g in 1:ngroups) {
 
         # extract variables in correct order
-        ov.idx  <- ov$idx[match(ov.names[[g]],   ov$name)]
+        if(!is.null(cluster) && length(cluster) > 0L) {
+            # keep 'joint' (Y,X) matrix in @X if multilevel (or always?)
+            OV.NAMES <- unique(c(ov.names[[g]], ov.names.x[[g]]))
+            ov.idx  <- ov$idx[match(OV.NAMES,   ov$name)]
+        } else {
+            ov.idx  <- ov$idx[match(ov.names[[g]],   ov$name)]
+        }
         exo.idx <- ov$idx[match(ov.names.x[[g]], ov$name)]
         all.idx <- unique(c(ov.idx, exo.idx))
 
@@ -866,10 +904,12 @@ lav_data_full <- function(data          = NULL,          # data.frame
             } else {
                 multilevel <- FALSE
             }
+            # ALWAYS add ov.names.x at the end, even if conditional.x (0.6-7)
+            OV.NAMES <- unique(c(ov.names[[g]], ov.names.x[[g]]))
             Lp[[g]] <- lav_data_cluster_patterns(Y = X[[g]], clus = clus,
                                                  cluster = cluster,
                                                  multilevel = multilevel,
-                                                 ov.names = ov.names[[g]],
+                                                 ov.names = OV.NAMES,
                                                  ov.names.l = ov.names.l[[g]])
 
             # new in 0.6-4
