@@ -29,18 +29,24 @@ computeSigmaHat <- function(lavmodel = NULL, GLIST = NULL, extra = FALSE,
         if(extra) {
             # check if matrix is positive definite
             ev <- eigen(Sigma.hat[[g]], symmetric=TRUE, only.values=TRUE)$values
-            if(any(ev < .Machine$double.eps) || sum(ev) == 0) {
+            if(any(ev < sqrt(.Machine$double.eps)) || sum(ev) == 0) {
                 Sigma.hat.inv <-  MASS::ginv(Sigma.hat[[g]])
                 Sigma.hat.log.det <- log(.Machine$double.eps)
                 attr(Sigma.hat[[g]], "po") <- FALSE
                 attr(Sigma.hat[[g]], "inv") <- Sigma.hat.inv
                 attr(Sigma.hat[[g]], "log.det") <- Sigma.hat.log.det
             } else {
-                ## FIXME
                 ## since we already do an 'eigen' decomposition, we should
-                ## 'reuse' that information, instead of doing a new cholesky
-                Sigma.hat.inv <-  inv.chol(Sigma.hat[[g]], logdet=TRUE)
+                ## 'reuse' that information, instead of doing a new cholesky?
+                # EV <- eigen(Sigma.hat[[g]], symmetric = TRUE)
+                # Sigma.hat.inv <- tcrossprod(EV$vectors / rep(EV$values,
+                #        each = length(EV$values)), EV$vectors)
+                # Sigma.hat.log.det <- sum(log(EV$values))
+
+                ## --> No, chol() is much (x2) times faster
+                Sigma.hat.inv <-  inv.chol(Sigma.hat[[g]], logdet = TRUE)
                 Sigma.hat.log.det <- attr(Sigma.hat.inv, "logdet")
+
                 attr(Sigma.hat[[g]], "po") <- TRUE
                 attr(Sigma.hat[[g]], "inv") <- Sigma.hat.inv
                 attr(Sigma.hat[[g]], "log.det") <- Sigma.hat.log.det
@@ -607,7 +613,7 @@ computeLAMBDA <- function(lavmodel = NULL, GLIST = NULL,
 }
 
 # THETA: observed (residual) variances
-computeTHETA <- function(lavmodel = NULL, GLIST = NULL) {
+computeTHETA <- function(lavmodel = NULL, GLIST = NULL, fix = TRUE) {
     # state or final?
     if(is.null(GLIST)) GLIST <- lavmodel@GLIST
 
@@ -625,11 +631,15 @@ computeTHETA <- function(lavmodel = NULL, GLIST = NULL) {
         MLIST <- GLIST[ mm.in.group ]
 
         if(representation == "LISREL") {
-            THETA.g <- computeTHETA.LISREL(MLIST = MLIST,
+            if(fix) {
+                THETA.g <- computeTHETA.LISREL(MLIST = MLIST,
                           ov.y.dummy.ov.idx = lavmodel@ov.y.dummy.ov.idx[[g]],
                           ov.x.dummy.ov.idx = lavmodel@ov.x.dummy.ov.idx[[g]],
                           ov.y.dummy.lv.idx = lavmodel@ov.y.dummy.lv.idx[[g]],
                           ov.x.dummy.lv.idx = lavmodel@ov.x.dummy.lv.idx[[g]])
+            } else {
+                THETA.g <- computeTHETA.LISREL(MLIST = MLIST)
+            }
         } else {
             stop("only representation LISREL has been implemented for now")
         }
@@ -640,6 +650,43 @@ computeTHETA <- function(lavmodel = NULL, GLIST = NULL) {
     THETA
 }
 
+# NU: observed intercepts
+computeNU <- function(lavmodel = NULL, GLIST = NULL,
+                      lavsamplestats = NULL) {
+    # state or final?
+    if(is.null(GLIST)) GLIST <- lavmodel@GLIST
+
+    nblocks        <- lavmodel@nblocks
+    nmat           <- lavmodel@nmat
+    representation <- lavmodel@representation
+
+    # return a list
+    NU <- vector("list", length=nblocks)
+
+    # compute NU for each group
+    for(g in 1:nblocks) {
+        # which mm belong to group g?
+        mm.in.group <- 1:nmat[g] + cumsum(c(0,nmat))[g]
+        MLIST <- GLIST[ mm.in.group ]
+
+        if(representation == "LISREL") {
+            NU.g <- computeNU.LISREL(MLIST = MLIST,
+                          sample.mean = lavsamplestats@mean[[g]],
+                          ov.y.dummy.ov.idx = lavmodel@ov.y.dummy.ov.idx[[g]],
+                          ov.x.dummy.ov.idx = lavmodel@ov.x.dummy.ov.idx[[g]],
+                          ov.y.dummy.lv.idx = lavmodel@ov.y.dummy.lv.idx[[g]],
+                          ov.x.dummy.lv.idx = lavmodel@ov.x.dummy.lv.idx[[g]])
+        } else {
+            stop("only representation LISREL has been implemented for now")
+        }
+
+        NU[[g]] <- NU.g
+    }
+
+    NU
+}
+
+
 # E(Y): expectation (mean) of observed variables
 # returns vector 1 x nvar
 computeEY <- function(lavmodel = NULL, GLIST = NULL, lavsamplestats = NULL) {
