@@ -27,15 +27,14 @@ nlminb.constr <- function(start, objective, gradient = NULL, hessian = NULL,
         if(is.null(ceq)) {
             ceq.jac <- function(x, ...) { matrix(0, nrow = 0L, ncol = length(x)) }
         } else {
-            ceq.jac <- function(x, ...) { lav_func_jacobian_complex(func = ceq, x = x, ...) }
+            ceq.jac <- function(x, ...) { numDeriv::jacobian(func = ceq, x = x, ...) }
         }
     }
     if(is.null(cin.jac)) {
         if(is.null(cin)) {
             cin.jac <- function(x, ...) { matrix(0, nrow = 0L, ncol = length(x)) }
         } else {
-            #require(numDeriv)
-            cin.jac <- function(x, ...) { lav_func_jacobian_complex(func = cin, x = x, ...) }
+            cin.jac <- function(x, ...) { numDeriv::jacobian(func = cin, x = x, ...) }
         }
     }
 
@@ -141,6 +140,7 @@ nlminb.constr <- function(start, objective, gradient = NULL, hessian = NULL,
         }
         optim.out <- nlminb(start = x.par, objective = auglag,
                             gradient = fgrad, control = control,
+                            lower = lower, upper = upper,
                             scale = scale, ...)
         ############################################################
         x.par <- optim.out$par
