@@ -13,6 +13,7 @@
 #                move syntax-based code to lav_syntax.R
 # - 26 April 2016: handle multiple 'blocks' (levels, classes, groups, ...)
 # - 24 March 2019: handle efa sets
+# - 23 May   2020: support for random slopes
 
 lavaanify <- lavParTable <- function(
 
@@ -132,7 +133,9 @@ lavaanify <- lavParTable <- function(
     }
 
     # check for meanstructure
-    if(any(FLAT$op == "~1")) meanstructure <- TRUE
+    if(any(FLAT$op == "~1")) {
+        meanstructure <- TRUE
+    }
 
     # check for block identifiers in the syntax (op = ":")
     n.block.flat <- length(which(FLAT$op == ":"))
@@ -198,6 +201,28 @@ lavaanify <- lavParTable <- function(
             FLAT.block <- FLAT[(BLOCK.op.idx[block]+1L):(BLOCK.op.idx[block+1]-1L),]
             # rm 'block' column (if any) in FLAT.block
             FLAT.block$block <- NULL
+
+            # new in 0.6-7: check for random slopes, add them here
+            if(block.lhs == "level" &&
+               block > 1L && # FIXME: multigroup,multilevel
+               !is.null(FLAT$rv) &&
+               any(nchar(FLAT$rv) > 0L)) {
+                lv.names.rv <- unique(FLAT$rv[nchar(FLAT$rv) > 0L])
+                for(i in 1:length(lv.names.rv)) {
+                    # add phantom latent variable
+                    TMP <- FLAT.block[1,]
+                    TMP$lhs <- lv.names.rv[i]; TMP$op <- "=~"
+                    TMP$rhs     <- lv.names.rv[i]
+                    TMP$mod.idx <- max(FLAT$mod.idx) + i
+                    TMP$fixed   <- "0"
+                    TMP$start   <- ""; TMP$lower   <- ""; TMP$upper   <- ""
+                    TMP$label   <- ""; TMP$prior   <- ""; TMP$efa     <- ""
+                    TMP$rv      <- lv.names.rv[i]
+                    FLAT.block <- rbind(FLAT.block, TMP, deparse.level = 0L)
+                    MOD <- c(MOD, list(list(fixed = 0)))
+                }
+            }
+
             LIST.block <- lav_partable_flat(FLAT.block, blocks = BLOCK.lhs,
                 block.id = block.id,
                 meanstructure = meanstructure,
@@ -238,7 +263,7 @@ lavaanify <- lavParTable <- function(
             block.rhs <- BLOCK.rhs[b]
             tmp <- try(scan(text = LIST[[block.lhs]], what = integer(),
                        quiet = TRUE), silent = TRUE)
-            if(class(tmp) == "integer") {
+            if(inherits(tmp, "integer")) {
                  LIST[[block.lhs]] <- tmp
             }
         }
@@ -304,8 +329,8 @@ lavaanify <- lavParTable <- function(
        LIST$ustart[scale.idx] <- 1
     }
 
-
     # apply user-specified modifiers
+    warn.about.single.label <- FALSE
     if(length(MOD)) {
         for(el in 1:length(MOD)) {
             idx <- which(LIST$mod.idx == el) # for each group
@@ -323,6 +348,7 @@ lavaanify <- lavParTable <- function(
             MOD.label <- MOD[[el]]$label
             MOD.prior <- MOD[[el]]$prior
             MOD.efa   <- MOD[[el]]$efa
+            MOD.rv    <- MOD[[el]]$rv
 
             # check for single argument if multiple groups
             if(ngroups > 1L && length(idx) > 1L) {
@@ -334,17 +360,31 @@ lavaanify <- lavParTable <- function(
                 if(length(MOD.upper) == 1L) MOD.upper <- rep(MOD.upper, ngroups)
                 if(length(MOD.prior) == 1L) MOD.prior <- rep(MOD.prior, ngroups)
                 if(length(MOD.efa)   == 1L) MOD.efa   <- rep(MOD.efa,   ngroups)
+                if(length(MOD.rv)    == 1L) MOD.rv    <- rep(MOD.rv,    ngroups)
+
+                # new in 0.6-7 (proposal):
+                # - always recycle modifiers, including labels
+                # - if ngroups > 1 AND group.label= is empty, produce a warning
+                #   (as this is a break from < 0.6-6)
+                if(length(MOD.label) == 1L) {
+                    MOD.label <- rep(MOD.label, ngroups)
+                    if(is.null(group.equal) || length(group.equal) == 0L) {
+                        warn.about.single.label <- TRUE
+                    }
+                }
+
+                # < 0.6-7 code:
                 # B) here we do NOT! otherwise, it would imply an equality
                 #                    constraint...
                 #    except if group.equal="loadings"!
-                if(length(MOD.label) == 1L) {
-                    if("loadings" %in% group.equal ||
-                       "composite.loadings" %in% group.equal) {
-                        MOD.label <- rep(MOD.label, ngroups)
-                    } else {
-                        MOD.label <- c(MOD.label, rep("", (ngroups-1L)) )
-                    }
-                }
+                #if(length(MOD.label) == 1L) {
+                #    if("loadings" %in% group.equal ||
+                #       "composite.loadings" %in% group.equal) {
+                #        MOD.label <- rep(MOD.label, ngroups)
+                #    } else {
+                #        MOD.label <- c(MOD.label, rep("", (ngroups-1L)) )
+                #    }
+                #}
             }
 
             # check for wrong number of arguments if multiple groups
@@ -355,6 +395,7 @@ lavaanify <- lavParTable <- function(
                 (!is.null(MOD.upper) && nidx != length(MOD.upper)) ||
                 (!is.null(MOD.prior) && nidx != length(MOD.prior)) ||
                 (!is.null(MOD.efa)   && nidx != length(MOD.efa))   ||
+                (!is.null(MOD.rv)    && nidx != length(MOD.rv))    ||
                 (!is.null(MOD.label) && nidx != length(MOD.label)) ) {
                 el.idx <- which(LIST$mod.idx == el)[1L]
                 stop("lavaan ERROR: wrong number of arguments in modifier (",
@@ -393,6 +434,16 @@ lavaanify <- lavParTable <- function(
                 }
                 LIST$efa[idx] <- MOD.efa
             }
+            if(!is.null(MOD.rv)) {
+                # do we already have a `rv' column? if not, create one
+                if(is.null(LIST$rv)) {
+                    LIST$rv <- character( length(LIST$lhs) )
+                }
+                LIST$rv[idx] <- MOD.rv
+
+                LIST$free[  idx] <- 0L
+                LIST$ustart[idx] <- as.numeric(NA) #
+            }
             if(!is.null(MOD.lower)) {
                 # do we already have a `lower' column? if not, create one
                 if(is.null(LIST$lower)) {
@@ -415,6 +466,37 @@ lavaanify <- lavParTable <- function(
     # remove mod.idx column
     LIST$mod.idx <- NULL
 
+    # warning about single label in multiple group setting?
+    if(warn.about.single.label) {
+        warning("lavaan WARNING: using a single label per parameter in a multiple group\n",  "\t setting implies imposing equality constraints across all the groups;\n",    "\t If this is not intended, either remove the label(s), or use a vector\n",    "\t of labels (one for each group);\n",
+             "\t See the Multiple groups section in the man page of model.syntax.")
+    }
+
+    # if lower/upper values were added, fix non-free values to ustart values
+    # new in 0.6-6
+    if(!is.null(LIST$lower)) {
+        fixed.idx <- which(LIST$free == 0L)
+        if(length(fixed.idx) > 0L) {
+            LIST$lower[fixed.idx] <- LIST$ustart[fixed.idx]
+        }
+    }
+    if(!is.null(LIST$upper)) {
+        fixed.idx <- which(LIST$free == 0L)
+        if(length(fixed.idx) > 0L) {
+            LIST$upper[fixed.idx] <- LIST$ustart[fixed.idx]
+        }
+    }
+
+    # if rv column is present, add RV.names to ALL rows where they are used
+    if(!is.null(LIST$rv)) {
+        RV.names <- unique(LIST$rv[ nchar(LIST$rv) > 0L ])
+        for(i in seq_len(length(RV.names))) {
+            lhs.idx <- which(LIST$lhs == RV.names[i] &
+                             LIST$op == "=~")
+            LIST$rv[lhs.idx] <- RV.names[i]
+        }
+    }
+
     if(debug) {
         cat("[lavaan DEBUG]: parameter LIST with MODIFIERS:\n")
         print( as.data.frame(LIST, stringsAsFactors=FALSE) )
@@ -437,6 +519,93 @@ lavaanify <- lavParTable <- function(
         print( as.data.frame(tmp, stringsAsFactors=FALSE) )
     }
 
+    # handle EFA equality constraints
+    # YR 14 Jan 2020: 0.6-6 does no longer impose 'explicit' constraints
+    #                 if we only need to fix a parameter to 0/1
+    # Note: we should also check if they are really needed:
+    #       eg., if all the factor-loadings of the 'second' set (time/group)
+    #       are constrained to be equal to the factor-loadings of the first
+    #       set, no further constraints are needed
+    if(auto.efa && !is.null(LIST$efa)) {
+        # for each set, for each block
+        nblocks <- lav_partable_nblocks(LIST)
+        set.names <- lav_partable_efa_values(LIST)
+        nsets <- length(set.names)
+
+        for(b in seq_len(nblocks)) {
+            for(s in seq_len(nsets)) {
+                # lv's for this block/set
+                lv.nam.efa <- unique(LIST$lhs[LIST$op == "=~" &
+                                              LIST$block == b &
+                                              LIST$efa == set.names[s]])
+                if(length(lv.nam.efa) == 1L) {
+                    # nothing to do (warn?)
+                    next
+                }
+
+                # equality constraints on ALL factor loadings in this set?
+                # --> no constraints are needed
+                #if(b > 1L || s > 1L) {
+                #    # collect plabels for this set, if any
+                #    set.idx <- which(LIST$op == "=~" &
+                #                     LIST$block == b &
+                #                     LIST$lhs %in% lv.nam.efa)
+                #    plabels.set <- LIST$plabel[ set.idx ]
+                #
+                #    # check simple == equalities
+                #    # TODO: what about more complex equality constraints?
+                #    eq.idx <- which(LIST$op == "==")
+                #    con.labels <- c(LIST$lhs[eq.idx],
+                #                    LIST$rhs[eq.idx])
+                #
+                #    # TODO: what about exceptions?
+                #    if(all(plabels.set %in% con.labels)) {
+                #        next
+                #    }
+                #}
+
+                # 1. echelon pattern
+                nfac <- length(lv.nam.efa)
+                for(f in seq_len(nfac)) {
+                    if(f == 1L) {
+                        next
+                    }
+                    nzero <- (f - 1L)
+                    ind.idx <- which(LIST$op == "=~" &
+                                     LIST$block == b &
+                                     LIST$lhs %in% lv.nam.efa[f])
+                    if(length(ind.idx) < nzero) {
+                        stop("lavaan ERROR: efa factor ", lv.nam.efa[f],
+                             " has not enough indicators for echelon pattern")
+                    }
+
+                    # fix to zero
+                    LIST$free[  ind.idx[seq_len(nzero)]] <- 0L
+                    LIST$ustart[ind.idx[seq_len(nzero)]] <- 0
+                    LIST$user[  ind.idx[seq_len(nzero)]] <- 7L
+
+                }
+
+                # 2. covariances constrained to zero (only if oblique rotation)
+                if(!orthogonal.efa) {
+                    # skip if user == 1 (user-override!)
+                    cov.idx <- which(LIST$op == "~~" &
+                                     LIST$block == b &
+                                     LIST$user == 0L &
+                                     LIST$lhs %in% lv.nam.efa &
+                                     LIST$rhs %in% lv.nam.efa &
+                                     LIST$lhs != LIST$rhs)
+
+                    # fix to zero
+                    LIST$free[  cov.idx] <- 0L
+                    LIST$ustart[cov.idx] <- 0
+                    LIST$user[  cov.idx] <- 7L
+                }
+
+            } # sets
+        } # blocks
+    } # auto.efa
+
     # handle user-specified equality constraints
     # lavaan 0.5-18
     # - rewrite 'LABEL-based' equality constraints as == constraints
@@ -453,6 +622,24 @@ lavaanify <- lavParTable <- function(
             all.idx <- which(LABEL == eq.label) # all same-label parameters
             ref.idx <- all.idx[1L]              # the first one only
 
+            # new in 0.6-6: make sure lower/upper constraints are equal too
+            if(!is.null(LIST$lower) &&
+               length(unique(LIST$lower[all.idx])) > 0L) {
+                non.inf <- which(is.finite(LIST$lower[all.idx]))
+                if(length(non.inf) > 0L) {
+                    smallest.val <- min(LIST$lower[all.idx][non.inf])
+                    LIST$lower[all.idx] <- smallest.val
+                }
+            }
+            if(!is.null(LIST$upper) &&
+                length(unique(LIST$upper[all.idx])) > 0L) {
+                non.inf <- which(is.finite(LIST$upper[all.idx]))
+                if(length(non.inf) > 0L) {
+                    largest.val <- max(LIST$upper[all.idx][non.inf])
+                    LIST$upper[all.idx] <- largest.val
+                }
+            }
+
             # two possibilities:
             # 1. all.idx contains a fixed parameter: in this case,
             #    we fix them all (hopefully to the same value)
@@ -513,14 +700,6 @@ lavaanify <- lavParTable <- function(
     }
 
 
-    # count free parameters
-    idx.free <- which(LIST$free > 0)
-    LIST$free[idx.free] <- seq_along(idx.free)
-    # backwards compatibility...
-    if(!is.null(LIST$unco)) {
-         LIST$unco[idx.free] <- seq_along(idx.free)
-    }
-
 
     # handle constraints (if any) (NOT per group, but overall - 0.4-11)
     if(length(CON) > 0L) {
@@ -574,97 +753,6 @@ lavaanify <- lavParTable <- function(
     LIST$label[def.idx] <- LIST$lhs[def.idx]
 
 
-    # handle EFA equality constraints
-    # Note: we should also check if they are really needed:
-    #       eg., if all the factor-loadings of the 'second' set (time/group)
-    #       are constrained to be equal to the factor-loadings of the first
-    #       set, no further constraints are needed
-    if(auto.efa && !is.null(LIST$efa)) {
-        # for each set, for each block
-        nblocks <- lav_partable_nblocks(LIST)
-        set.names <- lav_partable_efa_values(LIST)
-        nsets <- length(set.names)
-
-        plabel <- character(0L)
-        for(b in seq_len(nblocks)) {
-            for(s in seq_len(nsets)) {
-                # lv's for this block/set
-                lv.nam.efa <- unique(LIST$lhs[LIST$op == "=~" &
-                                              LIST$block == b &
-                                              LIST$efa == set.names[s]])
-                if(length(lv.nam.efa) == 1L) {
-                    # nothing to do (warn?)
-                    next
-                }
-
-                # equality constraints on ALL factor loadings in this set?
-                # --> no constraints are needed
-                #if(b > 1L || s > 1L) {
-                #    # collect plabels for this set, if any
-                #    set.idx <- which(LIST$op == "=~" &
-                #                     LIST$block == b &
-                #                     LIST$lhs %in% lv.nam.efa)
-                #    plabels.set <- LIST$plabel[ set.idx ]
-                #
-                #    # check simple == equalities
-                #    # TODO: what about more complex equality constraints?
-                #    eq.idx <- which(LIST$op == "==")
-                #    con.labels <- c(LIST$lhs[eq.idx],
-                #                    LIST$rhs[eq.idx])
-                #
-                #    # TODO: what about exceptions?
-                #    if(all(plabels.set %in% con.labels)) {
-                #        next
-                #    }
-                #}
-
-                # 1. echelon pattern
-                nfac <- length(lv.nam.efa)
-                for(f in seq_len(nfac)) {
-                    if(f == 1L) {
-                        next
-                    }
-                    nzero <- (f - 1L)
-                    ind.idx <- which(LIST$op == "=~" &
-                                     LIST$block == b &
-                                     LIST$lhs %in% lv.nam.efa[f])
-                    if(length(ind.idx) < nzero) {
-                        stop("lavaan ERROR: efa factor ", lv.nam.efa[f],
-                             " has not enough indicators for echelon pattern")
-                    }
-
-                    # add these plabels
-                    plabel <- c(plabel, LIST$plabel[ind.idx[seq_len(nzero)]])
-                }
-
-                # 2. covariances constrained to zero (only if oblique rotation)
-                if(!orthogonal.efa) {
-                    # skip if user == 1 (user-override!)
-                    cov.idx <- which(LIST$op == "~~" &
-                                     LIST$block == b &
-                                     LIST$user == 0L &
-                                     LIST$lhs %in% lv.nam.efa &
-                                     LIST$rhs %in% lv.nam.efa &
-                                     LIST$lhs != LIST$rhs)
-                    plabel <- c(plabel, LIST$plabel[cov.idx])
-                }
-
-            } # sets
-        } # blocks
-
-        # add zero constraints
-        nel <- length(plabel)
-        TMP <- list()
-        TMP$lhs    <- plabel
-        TMP$op     <- rep("==", nel)
-        TMP$rhs    <- rep(0,    nel)
-        TMP$block  <- rep(0L,   nel)
-        TMP$user   <- rep(7L,   nel) # special code for EFA == 7
-        TMP$ustart <- rep(as.numeric(NA), nel)
-
-        LIST <- lav_partable_merge(LIST, TMP)
-    }
-
     # handle effect.coding related equality constraints
     if(is.logical(effect.coding) && effect.coding) {
         effect.coding <- c("loadings", "intercepts")
@@ -765,7 +853,7 @@ lavaanify <- lavParTable <- function(
                         # free only if automatically added
                         if(length(lv.int.idx) > 0L &&
                            LIST$user[ lv.int.idx ] == 0L) {
-                            LIST$free[ lv.int.idx ] <- max(LIST$free) + 1L
+                            LIST$free[ lv.int.idx ] <- 1L
                         }
                     }
                 } # intercepts
@@ -828,7 +916,7 @@ lavaanify <- lavParTable <- function(
                 # free only if automatically added
                 if(length(lv.var.g1.idx) > 0L &&
                    LIST$user[ lv.var.g1.idx ] == 0L) {
-                    LIST$free[ lv.var.g1.idx ] <- max(LIST$free) + 1L
+                    LIST$free[ lv.var.g1.idx ] <- 1L
                 }
             }
         } # lv
@@ -888,7 +976,7 @@ lavaanify <- lavParTable <- function(
                 # free only if automatically added
                 if(length(lv.var.g1.idx) > 0L &&
                    LIST$user[ lv.var.g1.idx ] == 0L) {
-                    LIST$free[ lv.var.g1.idx ] <- max(LIST$free) + 1L
+                    LIST$free[ lv.var.g1.idx ] <- 1L
                 }
             }
         } # lv
@@ -897,6 +985,14 @@ lavaanify <- lavParTable <- function(
     }
 
 
+    # count free parameters
+    idx.free <- which(LIST$free > 0)
+    LIST$free[idx.free] <- seq_along(idx.free)
+    # backwards compatibility...
+    if(!is.null(LIST$unco)) {
+         LIST$unco[idx.free] <- seq_along(idx.free)
+    }
+
     if(debug) {
         cat("[lavaan DEBUG] lavParTable\n")
         print( as.data.frame(LIST) )
