@@ -13,13 +13,13 @@
 # overload standard R function `predict'
 setMethod("predict", "lavaan",
 function(object, newdata = NULL) {
-    lavPredict(object = object, newdata = newdata, type="lv", method="EBM",
-               fsm = FALSE,
-               optim.method = "bfgs")
+    lavPredict(object = object, newdata = newdata, type = "lv", method = "EBM",
+               fsm = FALSE, optim.method = "bfgs")
 })
 
 # main function
-lavPredict <- function(object, type = "lv", newdata = NULL, method = "EBM",
+lavPredict <- function(object, newdata = NULL, # keep order of predict(), 0.6-7
+                       type = "lv", method = "EBM",
                        se = "none", acov = "none", label = TRUE, fsm = FALSE,
                        append.data = FALSE, assemble = FALSE, # or TRUE?
                        level = 1L, optim.method = "bfgs", ETA = NULL) {
@@ -63,10 +63,10 @@ lavPredict <- function(object, type = "lv", newdata = NULL, method = "EBM",
             se <- acov <- "none"
             warning("lavaan WARNING: standard errors not available (yet) for non-normal data")
         }
-        if(lavdata@missing %in% c("ml", "ml.x")) {
-            se <- acov <- "none"
-            warning("lavaan WARNING: standard errors not available (yet) for missing data + fiml")
-        }
+        #if(lavdata@missing %in% c("ml", "ml.x")) {
+        #    se <- acov <- "none"
+        #    warning("lavaan WARNING: standard errors not available (yet) for missing data + fiml")
+        #}
     }
 
     # need full data set supplied
@@ -93,6 +93,19 @@ lavPredict <- function(object, type = "lv", newdata = NULL, method = "EBM",
                                               missing = lavdata@missing,
                                               warn = TRUE), # was FALSE before?
                            allow.single.case = TRUE)
+        # if ordered, check if number of levels is till the same (new in 0.6-7)
+        if(lavmodel@categorical) {
+            orig.ordered.idx <- which(lavdata@ov$type == "ordered")
+            orig.ordered.lev <- lavdata@ov$nlev[orig.ordered.idx]
+            match.new.idx <- match(newData@ov$name,
+                                   lavdata@ov$name[orig.ordered.idx])
+            new.ordered.lev <- newData@ov$nlev[match.new.idx]
+            if(any(orig.ordered.lev - new.ordered.lev != 0)) {
+                stop("lavaan ERROR: ",
+                     "mismatch number of categories for some ordered variables",
+                     "\n\t\tin newdata compared to original data.")
+            }
+        }
         data.obs <- newData@X
         eXo <- newData@eXo
         ov.names <- newData@ov.names
@@ -155,7 +168,6 @@ lavPredict <- function(object, type = "lv", newdata = NULL, method = "EBM",
 
 
         if(fsm) {
-            #FSM <- attr(out, "fsm")
             FSM <- lapply(seq_len(lavdata@ngroups), function(g) {
                        # determine block
                        if(lavdata@nlevels == 1L) {
@@ -169,7 +181,13 @@ lavPredict <- function(object, type = "lv", newdata = NULL, method = "EBM",
                                    lavmodel@ov.x.dummy.ov.idx[[bb]])
                        ret <- FSM[[g]]
                        if(length(lv.idx) > 0L) {
-                           ret <- FSM[[g]][-lv.idx, -ov.idx, drop=FALSE]
+                           if(is.matrix(FSM[[g]])) {
+                               ret <- FSM[[g]][-lv.idx, -ov.idx, drop = FALSE]
+                           } else if(is.list(FSM[[g]])) {
+                               FSM[[g]] <- lapply(FSM[[g]], function(x) {
+                                   ret <- x[-lv.idx, -ov.idx, drop = FALSE]
+                                   ret})
+                           }
                        }
                        ret
                    })
@@ -187,11 +205,11 @@ lavPredict <- function(object, type = "lv", newdata = NULL, method = "EBM",
                                    lavmodel@ov.x.dummy.lv.idx[[bb]])
                        ret <- SE[[g]]
                        if(length(lv.idx) > 0L) {
-                           ret <- SE[[g]][, -lv.idx, drop=FALSE]
+                           ret <- SE[[g]][, -lv.idx, drop = FALSE]
                        }
                        ret
                    })
-            if (acov != "none") {
+            if(acov != "none") {
               ACOV <- lapply(seq_len(lavdata@ngroups), function(g) {
                        # determine block
                        if (lavdata@nlevels == 1L) {
@@ -202,15 +220,19 @@ lavPredict <- function(object, type = "lv", newdata = NULL, method = "EBM",
                        lv.idx <- c(lavmodel@ov.y.dummy.lv.idx[[bb]],
                                    lavmodel@ov.x.dummy.lv.idx[[bb]])
                        ret <- ACOV[[g]]
-                       if (length(lv.idx) > 0L) {
-                           #FIXME: did I extrapolate this correctly?
-                           ret <- ACOV[[g]][-lv.idx, -lv.idx, drop=FALSE]
+                       if(length(lv.idx) > 0L) {
+                           if(is.matrix(ACOV[[g]])) {
+                               ret <- ACOV[[g]][-lv.idx, -lv.idx, drop = FALSE]
+                           } else if(is.list(ACOV[[g]])) {
+                               ACOV[[g]] <- lapply(ACOV[[g]], function(x) {
+                                   ret <- x[-lv.idx, -lv.idx, drop = FALSE]
+                                   ret})
+                           }
                        }
                        ret
                    })
-
-            }
-        }
+            } # acov
+        } # se
 
         # label?
         if(label) {
@@ -228,13 +250,32 @@ lavPredict <- function(object, type = "lv", newdata = NULL, method = "EBM",
                     colnames(out[[g]]) <- lavpta$vnames$lv[[gg]]
                 }
 
+                if(fsm) {
+                    if(is.matrix(FSM[[g]])) {
+                        dimnames(FSM[[g]]) <- list(lavpta$vnames$lv[[gg]],
+                                                   ov.names[[g]]) # !not gg
+                    } else if(is.list(FSM[[g]])) {
+                        FSM[[g]] <- lapply(FSM[[g]], function(x) {
+                            dimnames(x) <- list(lavpta$vnames$lv[[gg]],
+                                                ov.names[[g]]) # !not gg
+                            x})
+                    }
+                }
+
                 if(se != "none") {
                     colnames(SE[[g]]) <- lavpta$vnames$lv[[gg]]
                 }
 
                 if(acov != "none") {
-                    dimnames(ACOV[[g]]) <- list(lavpta$vnames$lv[[gg]],
+                    if(is.matrix(ACOV[[g]])) {
+                        dimnames(ACOV[[g]]) <- list(lavpta$vnames$lv[[gg]],
+                                                    lavpta$vnames$lv[[gg]])
+                    } else if(is.list(ACOV[[g]])) {
+                        ACOV[[g]] <- lapply(ACOV[[g]], function(x) {
+                            dimnames(x) <- list(lavpta$vnames$lv[[gg]],
                                                 lavpta$vnames$lv[[gg]])
+                            x})
+                    }
                 }
 
             } # g
@@ -469,7 +510,9 @@ lav_predict_eta_normal <- function(lavobject = NULL,  # for convenience
         FSM <- vector("list", length = lavdata@ngroups)
     }
 
-    if (acov != "none") se <- acov # ACOV implies SE
+    if (acov != "none") {
+        se <- acov # ACOV implies SE
+    }
     if(se != "none") {
         SE <- vector("list", length = lavdata@ngroups)
         # return full sampling covariance matrix?
@@ -539,6 +582,14 @@ lav_predict_eta_normal <- function(lavobject = NULL,  # for convenience
         # center data
         Yc <- t( t(data.obs.g) - EY.g )
 
+        # global factor score coefficient matrix 'C'
+        FSC <- VETA.g %*% t(LAMBDA.g) %*% Sigma.inv.g
+
+        # store fsm?
+        if(fsm) {
+            FSM.g <- FSC
+        }
+
         # compute factor scores
         if(lavdata@missing %in% c("ml", "ml.x")) {
 
@@ -548,6 +599,18 @@ lav_predict_eta_normal <- function(lavobject = NULL,  # for convenience
             # factor scores container
             FS.g <- matrix(as.numeric(NA), nrow(Yc), ncol = length(EETA.g))
 
+            #if(fsm) {
+            #    FSM.g <- vector("list", length = Mp$npatterns)
+            #}
+
+            if(se == "standard") {
+                SE.g <- matrix(as.numeric(NA), nrow(Yc), ncol = length(EETA.g))
+            }
+
+            if (acov == "standard") {
+                ACOV.g <- vector("list", length = Mp$npatterns)
+            }
+
             # compute FSC per pattern
             for(p in seq_len(Mp$npatterns)) {
                 var.idx <- Mp$pat[p,]     # observed
@@ -567,23 +630,38 @@ lav_predict_eta_normal <- function(lavobject = NULL,  # for convenience
                 lambda <- LAMBDA.g[var.idx, , drop = FALSE]
                 FSC <- VETA.g %*% t(lambda) %*% Sigma_22.inv
 
+                # FSM?
+                #if(fsm) {
+                #    tmp <- matrix(as.numeric(NA), nrow = ncol(lambda),
+                #                  ncol = ncol(Yc))
+                #    tmp[,var.idx] <- FSC
+                #    FSM.g[[p]] <- tmp
+                #}
+
+                # factor score for this pattern
                 FS.g[Mp$case.idx[[p]], ] <- t(FSC %*% t(Oc) + EETA.g)
-            }
 
-            # what about FSM? There is no single one, but as many as patterns
-            if(fsm) {
-                # use 'global' version (just like in complete case)
-                FSM[[g]] <- VETA.g %*% t(LAMBDA.g) %*% Sigma.inv.g
-            }
+                # SE?
+                if(se == "standard") {
+                    tmp <- (VETA.g - VETA.g %*% t(lambda) %*%
+                                     Sigma_22.inv %*% lambda %*% VETA.g)
+                    tmp.d <- diag(tmp)
+                    tmp.d[ tmp.d < 1e-05 ] <- as.numeric(NA)
+
+                    # all cases in this pattern get the same SEs
+                    SE.g[Mp$case.idx[[p]], ] <- matrix(sqrt(tmp.d),
+                             nrow = length(Mp$case.idx[[p]]),
+                             ncol = ncol(SE.g), byrow = TRUE)
+                }
+
+                # ACOV?
+                if(acov == "standard") {
+                    ACOV.g[[p]] <- tmp # for this pattern
+                }
+
+            } # p
+
         } else {
-            # factor score coefficient matrix 'C'
-            FSC <- VETA.g %*% t(LAMBDA.g) %*% Sigma.inv.g
-
-            # store fsm?
-            if(fsm) {
-                FSM[[g]] <- FSC
-            }
-
             # compute factor scores
             FS.g <- t(FSC %*% t(Yc) + EETA.g)
         }
@@ -600,20 +678,32 @@ lav_predict_eta_normal <- function(lavobject = NULL,  # for convenience
 
         FS[[g]] <- FS.g
 
-        # standard error
-        if(se == "standard" && !(lavdata@missing %in% c("ml", "ml.x"))) {
-            tmp <- (VETA.g -
-             VETA.g %*% t(LAMBDA.g) %*% Sigma.inv.g %*% LAMBDA.g %*% VETA.g)
-            tmp.d <- diag(tmp)
-            tmp.d[ tmp.d < 1e-05 ] <- as.numeric(NA)
-            SE[[g]] <- matrix(sqrt(tmp.d), nrow = 1L)
-
-            # return full sampling covariance matrix?
-            if (acov == "standard") {
-              ACOV[[g]] <- tmp
-            }
+        # FSM
+        if(fsm) {
+            FSM[[g]] <- FSM.g
         }
-    }
+
+        # standard error
+        if(se == "standard") {
+            if(lavdata@missing %in% c("ml", "ml.x")) {
+                SE[[g]] <- SE.g
+                if(acov == "standard") {
+                    ACOV[[g]] <- ACOV.g
+                }
+            } else { # complete data
+                tmp <- (VETA.g - VETA.g %*% t(LAMBDA.g) %*%
+                                  Sigma.inv.g %*% LAMBDA.g %*% VETA.g)
+                tmp.d <- diag(tmp)
+                tmp.d[ tmp.d < 1e-05 ] <- as.numeric(NA)
+                SE[[g]] <- matrix(sqrt(tmp.d), nrow = 1L)
+
+                # return full sampling covariance matrix?
+                if (acov == "standard") {
+                  ACOV[[g]] <- tmp
+                }
+            }
+        } # se = "standard"
+    } # g
 
     if(fsm) {
         attr(FS, "fsm") <- FSM
@@ -770,6 +860,16 @@ lav_predict_eta_bartlett <- function(lavobject = NULL, # for convenience
         # center data
         Yc <- t( t(data.obs.g) - EY.g )
 
+        # global factor score coefficient matrix 'C'
+        FSC <- ( MASS::ginv(t(LAMBDA.g) %*% Sigma.inv.g %*% LAMBDA.g)
+                 %*% t(LAMBDA.g) %*% Sigma.inv.g )
+
+        # store fsm?
+        if(fsm) {
+            # store fsm?
+            FSM.g <- FSC
+        }
+
         # compute factor scores
         if(lavdata@missing %in% c("ml", "ml.x")) {
 
@@ -779,6 +879,18 @@ lav_predict_eta_bartlett <- function(lavobject = NULL, # for convenience
             # factor scores container
             FS.g <- matrix(as.numeric(NA), nrow(Yc), ncol = length(EETA.g))
 
+            #if(fsm) {
+            #    FSM.g <- vector("list", length = Mp$npatterns)
+            #}
+
+            if(se == "standard") {
+                SE.g <- matrix(as.numeric(NA), nrow(Yc), ncol = length(EETA.g))
+            }
+
+            if (acov == "standard") {
+                ACOV.g <- vector("list", length = Mp$npatterns)
+            }
+
             # compute FSC per pattern
             for(p in seq_len(Mp$npatterns)) {
                 var.idx <- Mp$pat[p,]     # observed
@@ -798,25 +910,45 @@ lav_predict_eta_bartlett <- function(lavobject = NULL, # for convenience
                 lambda <- LAMBDA.g[var.idx, , drop = FALSE]
                 FSC <- ( MASS::ginv(t(lambda) %*% Sigma_22.inv %*% lambda)
                            %*% t(lambda) %*% Sigma_22.inv )
+
+                # FSM?
+                #if(fsm) {
+                #    tmp <- matrix(as.numeric(NA), nrow = ncol(lambda),
+                #                  ncol = ncol(Yc))
+                #    tmp[,var.idx] <- FSC
+                #    FSM.g[[p]] <- tmp
+                #}
+
+                # factor scores for this pattern
                 FS.g[Mp$case.idx[[p]], ] <- t(FSC %*% t(Oc) + EETA.g)
+
+                # SE?
+                if(se == "standard") {
+                    tmp <- ( MASS::ginv(t(lambda) %*% Sigma_22.inv %*% lambda)
+                             - VETA.g )
+                    tmp.d <- diag(tmp)
+                    tmp.d[ tmp.d < 1e-05 ] <- as.numeric(NA)
+
+                    # all cases in this pattern get the same SEs
+                    SE.g[Mp$case.idx[[p]], ] <- matrix(sqrt(tmp.d),
+                             nrow = length(Mp$case.idx[[p]]),
+                             ncol = ncol(SE.g), byrow = TRUE)
+                }
+
+                # ACOV?
+                if(acov == "standard") {
+                    ACOV.g[[p]] <- tmp # for this pattern
+                }
             }
 
             # what about FSM? There is no single one, but as many as patterns
-            if(fsm) {
-                # use 'global' version (just like in complete case)
-                FSM[[g]] <- ( MASS::ginv(t(LAMBDA.g) %*% Sigma.inv.g %*%
-                                LAMBDA.g) %*% t(LAMBDA.g) %*% Sigma.inv.g )
-            }
+            #if(fsm) {
+            #    # use 'global' version (just like in complete case)
+            #    FSM[[g]] <- ( MASS::ginv(t(LAMBDA.g) %*% Sigma.inv.g %*%
+            #                    LAMBDA.g) %*% t(LAMBDA.g) %*% Sigma.inv.g )
+            #}
 
         } else {
-            # factor score coefficient matrix 'C'
-            FSC <- ( MASS::ginv(t(LAMBDA.g) %*% Sigma.inv.g %*% LAMBDA.g)
-                     %*% t(LAMBDA.g) %*% Sigma.inv.g )
-            # store fsm?
-            if(fsm) {
-                FSM[[g]] <- FSC
-            }
-
             # compute factor scores
             FS.g <- t(FSC %*% t(Yc) + EETA.g)
         }
@@ -833,28 +965,40 @@ lav_predict_eta_bartlett <- function(lavobject = NULL, # for convenience
 
         FS[[g]] <- FS.g
 
-        # standard error
-        if(se == "standard" && !(lavdata@missing %in% c("ml", "ml.x"))) {
-            # the traditional formula is:
-            #     solve(t(lambda) %*% solve(theta) %*% lambda)
-            # but we replace it by
-            #     solve( t(lambda) %*% solve(sigma) %*% lambda ) - psi
-            # to handle negative variances
-            # in addition, we use ginv
-            tmp <- ( MASS::ginv(t(LAMBDA.g) %*%
-                                  Sigma.inv.g %*% LAMBDA.g)
-                     - VETA.g )
-            tmp.d <- diag(tmp)
-            tmp.d[ tmp.d < 1e-05 ] <- as.numeric(NA)
-            SE[[g]] <- matrix(sqrt(tmp.d), nrow = 1L)
-
-            # return full sampling covariance matrix?
-            if (acov == "standard") {
-              ACOV[[g]] <- tmp
-            }
+        # FSM
+        if(fsm) {
+            FSM[[g]] <- FSM.g
         }
 
-    }
+        # standard error
+        if(se == "standard") {
+            if(lavdata@missing %in% c("ml", "ml.x")) {
+                SE[[g]] <- SE.g
+                if(acov == "standard") {
+                    ACOV[[g]] <- ACOV.g
+                }
+            } else { # complete data
+
+                # the traditional formula is:
+                #     solve(t(lambda) %*% solve(theta) %*% lambda)
+                # but we replace it by
+                #     solve( t(lambda) %*% solve(sigma) %*% lambda ) - psi
+                # to handle negative variances
+                # in addition, we use ginv
+                tmp <- ( MASS::ginv(t(LAMBDA.g) %*% Sigma.inv.g %*% LAMBDA.g)
+                         - VETA.g )
+                tmp.d <- diag(tmp)
+                tmp.d[ tmp.d < 1e-05 ] <- as.numeric(NA)
+                SE[[g]] <- matrix(sqrt(tmp.d), nrow = 1L)
+
+                # return full sampling covariance matrix?
+                if (acov == "standard") {
+                  ACOV[[g]] <- tmp
+                }
+            }
+        } # se
+
+    } # g
 
     if(fsm) {
         attr(FS, "fsm") <- FSM
@@ -923,7 +1067,7 @@ lav_predict_eta_ebm_ml <- function(lavobject = NULL,  # for convenience
         }
     }
     EETAx <- computeEETAx(lavmodel = lavmodel, lavsamplestats = lavsamplestats,
-                          eXo = eXo, nobs = lavdata@norig,
+                          eXo = eXo, nobs = lapply(data.obs, NROW),
                           remove.dummy.lv = TRUE) ## FIXME?
     TH    <- computeTH(   lavmodel = lavmodel)
     THETA <- computeTHETA(lavmodel = lavmodel)
@@ -1025,6 +1169,9 @@ lav_predict_eta_ebm_ml <- function(lavobject = NULL,  # for convenience
             mu.i <- EETAx[[g]][i,,drop=FALSE]
             y.i <- data.obs[[g]][i,,drop=FALSE]
 
+            ### DEBUG ONLY:
+            #cat("i = ", i, "mu.i = ", mu.i, "\n")
+
             START <- numeric(nfac) # initial values for eta
 
             if(!all(is.na(y.i))) {
@@ -1138,7 +1285,7 @@ lav_predict_yhat <- function(lavobject = NULL, # for convience
 
     YHAT <- computeYHAT(lavmodel = lavmodel, GLIST = NULL,
                         lavsamplestats = lavsamplestats, eXo = eXo,
-                        nobs = lavdata@norig,
+                        nobs = lapply(data.obs, NROW),
                         ETA = ETA, duplicate = duplicate)
 
     # if conditional.x, paste eXo
@@ -1250,11 +1397,6 @@ lav_predict_fy_internal <- function(X = NULL, yhat = NULL,
 
     # first, NUMERIC variables
     if(length(num.idx) > 0L) {
-        # multivariate
-        # FY.group[,num.idx] <-
-        #    dmnorm(X[,num.idx],
-        #           mean = yhat[n,num.idx],
-            #           varcov = THETA[[g]][num.idx, num.idx], log = log.)
         for(v in num.idx) {
             FY.group[,v] <- dnorm(X[,v],
                                   # YHAT may change or not per case
