@@ -12,6 +12,8 @@ lav_start <- function(start.method    = "default",
                       lavsamplestats  = NULL,
                       model.type      = "sem",
                       mimic           = "lavaan",
+                      reflect         = FALSE,    # rotation only
+                      order.lv.by     = "none",   # rotation only
                       debug           = FALSE) {
 
     # check arguments
@@ -21,26 +23,21 @@ lav_start <- function(start.method    = "default",
     categorical <- any(lavpartable$op == "|")
 
     # conditional.x?
-    conditional.x <- any(lavpartable$exo == 1L & lavpartable$op == "~")
+    conditional.x <- any(lavpartable$exo == 1L &
+                         lavpartable$op %in% c("~", "<~"))
     #ord.names <- unique(lavpartable$lhs[ lavpartable$op == "|" ])
 
     # nlevels?
     nlevels <- lav_partable_nlevels(lavpartable)
 
-    # shortcut for 'simple'
-    # be we should also take care of the 'fixed.x' issue
-    if(identical(start.method, "simple")) {
-        start <- numeric( length(lavpartable$ustart) )
-        start[ which(lavpartable$op == "=~") ] <- 1.0
-        start[ which(lavpartable$op == "~*~") ] <- 1.0
-        ov.names.ord <- vnames(lavpartable, "ov.ord")
-        var.idx <- which(lavpartable$op == "~~" & lavpartable$lhs == lavpartable$rhs &
-                         !(lavpartable$lhs %in% ov.names.ord))
-        start[var.idx] <- 1.0
-        user.idx <- which(!is.na(lavpartable$ustart))
-        start[user.idx] <- lavpartable$ustart[user.idx]
-        return(start)
+    # reflect/order.lv.by
+    if(is.null(reflect)) {
+        reflect <- FALSE
     }
+    if(is.null(order.lv.by)) {
+        order.lv.by <- "index"
+    }
+
 
     # check start.method
     if(mimic == "lavaan") {
@@ -51,11 +48,27 @@ lav_start <- function(start.method    = "default",
         # FIXME: use LISREL/EQS/AMOS/.... schems
         start.initial <- "lavaan"
     }
-    start.user    <- NULL
+
+    # start.method
+    start.user <- NULL
     if(is.character(start.method)) {
         start.method. <- tolower(start.method)
         if(start.method. == "default") {
             # nothing to do
+        } else if(start.method == "simple") {
+            start <- numeric( length(lavpartable$ustart) )
+            start[ which(lavpartable$op == "=~") ] <- 1.0
+            start[ which(lavpartable$op == "~*~") ] <- 1.0
+            ov.names.ord <- vnames(lavpartable, "ov.ord")
+            var.idx <- which(lavpartable$op == "~~" &
+                             lavpartable$lhs == lavpartable$rhs &
+                             !(lavpartable$lhs %in% ov.names.ord))
+            start[var.idx] <- 1.0
+            user.idx <- which(!is.na(lavpartable$ustart))
+            start[user.idx] <- lavpartable$ustart[user.idx]
+            return(start)
+        } else if(start.method == "est") {
+            return(lavpartable$est)
         } else if(start.method. %in% c("simple", "lavaan", "mplus")) {
             start.initial <- start.method.
         } else {
@@ -66,6 +79,7 @@ lav_start <- function(start.method    = "default",
     } else if(inherits(start.method, "lavaan")) {
         start.user <- parTable(start.method)
     }
+
     # check model list elements, if provided
     if(!is.null(start.user)) {
         if(is.null(start.user$lhs) ||
@@ -93,8 +107,7 @@ lav_start <- function(start.method    = "default",
 
     # 1. =~ factor loadings:
     if(categorical) {
-        # if std.lv=TRUE, more likely initial Sigma.hat is positive definite
-        # 0.8 is too large
+        # if std.lv=TRUE, 0.8 is too large
         start[ which(lavpartable$op == "=~") ] <- 0.7
     } else {
         start[ which(lavpartable$op == "=~") ] <- 1.0
@@ -130,7 +143,7 @@ lav_start <- function(start.method    = "default",
         group.values <- lav_partable_group_values(lavpartable)
 
         # info from user model for this group
-        if(conditional.x) {
+        if(conditional.x && nlevels == 1L) {
             ov.names     <- vnames(lavpartable, "ov.nox", group = group.values[g])
         } else {
             ov.names     <- vnames(lavpartable, "ov", group = group.values[g])
@@ -141,14 +154,16 @@ lav_start <- function(start.method    = "default",
         } else {
             ov.names.num <- ov.names
         }
-        lv.names    <- vnames(lavpartable, "lv",   group = group.values[g])
-        ov.names.x  <- vnames(lavpartable, "ov.x", group = group.values[g])
+        lv.names     <- vnames(lavpartable, "lv",     group = group.values[g])
+        lv.names.efa <- vnames(lavpartable, "lv.efa", group = group.values[g])
+        ov.names.x   <- vnames(lavpartable, "ov.x",   group = group.values[g])
 
         # just for the nlevels >1 case
-        ov.names <- unique(unlist(ov.names))
+        ov.names     <- unique(unlist(ov.names))
         ov.names.num <- unique(unlist(ov.names.num))
-        lv.names <- unique(unlist(lv.names))
-        ov.names.x <- unique(unlist(ov.names.x))
+        lv.names     <- unique(unlist(lv.names))
+        lv.names.efa <- unique(unlist(lv.names.efa))
+        ov.names.x   <- unique(unlist(ov.names.x))
 
 
         # residual ov variances (including exo/ind, to be overriden)
@@ -161,7 +176,7 @@ lav_start <- function(start.method    = "default",
             start[ov.var.idx] <- diag(lavsamplestats@cov[[g]])[sample.var.idx]
         } else {
             if(start.initial == "mplus") {
-                if(conditional.x) {
+                if(conditional.x && nlevels == 1L) {
                     start[ov.var.idx] <-
                        (1.0 - 0.50)*lavsamplestats@res.var[[1L]][sample.var.idx]
                 } else {
@@ -169,7 +184,7 @@ lav_start <- function(start.method    = "default",
                        (1.0 - 0.50)*lavsamplestats@var[[1L]][sample.var.idx]
                 }
             } else {
-                if(conditional.x) {
+                if(conditional.x && nlevels == 1L) {
                     start[ov.var.idx] <-
                   (1.0 - 0.50)*diag(lavsamplestats@res.cov[[g]])[sample.var.idx]
                 } else {
@@ -184,6 +199,10 @@ lav_start <- function(start.method    = "default",
            model.type %in% c("sem", "cfa") ) {
             # fabin3 estimator (2sls) of Hagglund (1982) per factor
             for(f in lv.names) {
+                # not for efa factors
+                if(f %in% lv.names.efa) {
+                    next
+                }
                 lambda.idx <- which( lavpartable$lhs == f &
                                      lavpartable$op == "=~" &
                                      lavpartable$group == group.values[g] )
@@ -209,7 +228,7 @@ lav_start <- function(start.method    = "default",
                         COV <- lavsamplestats@missing.h1[[g]]$sigma[ov.idx,
                                                       ov.idx, drop = FALSE]
                     } else {
-                        if(conditional.x) {
+                        if(conditional.x && nlevels == 1L) {
                             COV <- lavsamplestats@res.cov[[g]][ov.idx,
                                                       ov.idx, drop = FALSE]
                         } else {
@@ -224,7 +243,9 @@ lav_start <- function(start.method    = "default",
                                                 method = "fabin3")
 
                     # factor loadings
-                    start[lambda.idx] <- fabin$lambda
+                    tmp <- fabin$lambda
+                    tmp[ !is.finite(tmp) ] <- 1.0 # just in case (eg 0/0)
+                    start[lambda.idx] <- tmp
 
                     # factor variance
                     #if(!std.lv) {
@@ -261,8 +282,73 @@ lav_start <- function(start.method    = "default",
                     #    start[res.idx][neg.idx] <- 0.05
                     #}
                 }
-            }
-        } # fabin
+            } # fabin3
+
+            # efa?
+            nefa <- lav_partable_nefa(lavpartable)
+            if(nefa > 0L) {
+                efa.values <- lav_partable_efa_values(lavpartable)
+
+                for(set in seq_len(nefa)) {
+                    # determine ov idx for this set
+                    ov.efa <-
+                        unique(lavpartable$rhs[ lavpartable$op == "=~" &
+                                                lavpartable$block == g &
+                                                lavpartable$efa == efa.values[set]])
+                    lv.efa <-
+                        unique(lavpartable$lhs[ lavpartable$op == "=~" &
+                                                lavpartable$block == g &
+                                                lavpartable$efa == efa.values[set]])
+                    lambda.idx <- which( lavpartable$lhs %in% lv.efa &
+                                         lavpartable$op == "=~" &
+                                         lavpartable$group == group.values[g] )
+
+                    theta.idx <- which( lavpartable$lhs %in% ov.efa &
+                                        lavpartable$op == "~~" &
+                                        lavpartable$lhs == lavpartable$rhs &
+                                        lavpartable$group == group.values[g] )
+
+                    # get observed indicators for these EFA lv variables
+                    ov.idx <- match(unique(lavpartable$rhs[lambda.idx]),
+                                    ov.names)
+
+                    if(length(ov.idx) > 0L && !any(is.na(ov.idx))) {
+                        if(lavsamplestats@missing.flag && nlevels == 1L) {
+                            COV <- lavsamplestats@missing.h1[[g]]$sigma[ov.idx,
+                                                      ov.idx, drop = FALSE]
+                        } else {
+                            if(conditional.x && nlevels == 1L) {
+                                COV <- lavsamplestats@res.cov[[g]][ov.idx,
+                                                          ov.idx, drop = FALSE]
+                            } else {
+                                COV <- lavsamplestats@cov[[g]][ov.idx,
+                                                          ov.idx, drop = FALSE]
+                            }
+                         }
+
+                        # EFA solution with zero upper-right corner
+                        EFA <- lav_efa_extraction(S = COV,
+                                              nfactors = length(lv.efa),
+                                              method = "ML",
+                                              order.lv.by = order.lv.by,
+                                              #order.lv.by = "none",
+                                              #reflect = reflect,
+                                              reflect = FALSE,
+                                              corner = TRUE)
+
+                        # factor loadings
+                        tmp <- as.numeric(EFA$LAMBDA)
+                        tmp[ !is.finite(tmp) ] <- 1.0 # just in case (eg 0/0)
+                        start[lambda.idx] <- tmp
+
+                        # residual variances
+                        tmp <- diag(EFA$THETA)
+                        tmp[ !is.finite(tmp) ] <- 1.0 # just in case
+                        start[theta.idx] <- tmp
+                    }
+                } # set
+            } # efa
+        } # factor loadings
 
         if(model.type == "unrestricted") {
            # fill in 'covariances' from lavsamplestats
@@ -291,7 +377,7 @@ lav_start <- function(start.method    = "default",
         if(lavsamplestats@missing.flag && nlevels == 1L) {
             start[ov.int.idx] <- lavsamplestats@missing.h1[[g]]$mu[sample.int.idx]
         } else {
-            if(conditional.x) {
+            if(conditional.x && nlevels == 1L) {
                 start[ov.int.idx] <- lavsamplestats@res.int[[g]][sample.int.idx]
             } else {
                 start[ov.int.idx] <- lavsamplestats@mean[[g]][sample.int.idx]
@@ -307,7 +393,7 @@ lav_start <- function(start.method    = "default",
                 lavsamplestats@th.names[[g]][ lavsamplestats@th.idx[[g]] > 0L ]
             # th.names.sample should identical to
             # vnames(lavpartable, "th", group = group.values[g])
-            if(conditional.x) {
+            if(conditional.x && nlevels == 1L) {
                 th.values <-
                  lavsamplestats@res.th[[g]][lavsamplestats@th.idx[[g]] > 0L]
             } else {
@@ -361,7 +447,7 @@ lav_start <- function(start.method    = "default",
         if(length(lv.x) > 0L) {
             for(ll in lv.x) {
                 ind.idx <- which(lavpartable$op == "=~" &
-                                 lavpartable$lhs == ll,
+                                 lavpartable$lhs == ll  &
                                  lavpartable$group == group.values[g])
                 if(length(ind.idx) == 1L) {
                     single.ind <- lavpartable$rhs[ind.idx]
@@ -381,7 +467,7 @@ lav_start <- function(start.method    = "default",
                          single.var <- 1
                     }
                     ov.idx <- match(single.ind, ov.names)
-                    if(conditional.x) {
+                    if(conditional.x && nlevels == 1L) {
                         ov.var <- diag(lavsamplestats@res.cov[[g]])[ov.idx]
                     } else {
                         ov.var <- diag(lavsamplestats@cov[[g]])[ov.idx]
@@ -467,15 +553,8 @@ lav_start <- function(start.method    = "default",
       #      } # free.var.idx
       #  } # lv var
 
-    } # groups
-
-
-    # nlevels > 1L
-    if(nlevels > 1L) {
-        for(g in 1:ngroups) {
-            group.values <- lav_partable_group_values(lavpartable)
-            ov.names.x  <- vnames(lavpartable, "ov.x", group = group.values[g])
-            ov.names.x <- unique(unlist(ov.names.x))
+        # nlevels > 1L
+        if(nlevels > 1L) {
             level.values <- lav_partable_level_values(lavpartable)
 
             if(!conditional.x && length(ov.names.x) > 0) {
@@ -512,8 +591,12 @@ lav_start <- function(start.method    = "default",
                     start[ov.int.idx] <- INT[idx]
                 } # levels
             } # fixed.x
-        } # groups
-    } # nlevels > 1L
+        } # nlevels > 1L
+
+
+    } # groups
+
+
 
 
     # group weights
@@ -544,6 +627,22 @@ lav_start <- function(start.method    = "default",
         #                    lavpartable$lhs %in% lv.names)
     }
 
+
+    # adjust if outside bounds -- new in 0.6-6
+    if(!is.null(lavpartable$lower)) {
+        bad.idx <- which(start < lavpartable$lower)
+        if(length(bad.idx)) {
+            start[bad.idx] <- lavpartable$lower[bad.idx]
+        }
+    }
+    if(!is.null(lavpartable$upper)) {
+        bad.idx <- which(start > lavpartable$upper)
+        if(length(bad.idx)) {
+            start[bad.idx] <- lavpartable$upper[bad.idx]
+        }
+    }
+
+
     # override if a user list with starting values is provided
     # we only look at the 'est' column for now
     if(!is.null(start.user)) {
@@ -575,9 +674,20 @@ lav_start <- function(start.method    = "default",
     }
 
     # override if the model syntax contains explicit starting values
-    user.idx <- which(!is.na(lavpartable$ustart))
+    user.idx <- which(!is.na(lavpartable$ustart) &
+                      lavpartable$user != 7L) # new in 0.6-7, if rotation and
+                                              # and we change the order of lv's
     start[user.idx] <- lavpartable$ustart[user.idx]
 
+    # final check: no NaN or other non-finite values
+    bad.idx <- which(!is.finite(start))
+    if(length(bad.idx) > 0L) {
+        cat("starting values:\n")
+        print( start )
+        warning("lavaan WARNING: some starting values are non-finite; replacing them with 0.5; please provide better starting values.\n")
+        start[ bad.idx ] <- 0.5
+    }
+
     if(debug) {
         cat("lavaan DEBUG: lavaanStart\n")
         print( start )
@@ -662,6 +772,7 @@ lav_start_check_cov <- function(lavpartable = NULL, start = lavpartable$start) {
 
             # check
             COR <- start[this.cov.idx] / sqrt(var.lhs.value * var.rhs.value)
+            # NOTE: we treat this as an unconditional COR!
 
             if(!is.finite(COR)) {
                 # force simple values
@@ -672,8 +783,7 @@ lav_start_check_cov <- function(lavpartable = NULL, start = lavpartable$start) {
                 start[var.rhs.idx] <- 1
                 start[this.cov.idx] <- 0
             } else if(abs(COR) > 1) {
-                warning(
-  "lavaan WARNING: starting values imply a correlation larger than 1;\n",
+                txt <- paste("lavaan WARNING: starting values imply a correlation larger than 1;\n",
 "                  variables involved are: ", var.lhs, " ", var.rhs, block.txt)
 
                 # three ways to fix it: rescale cov12, var1 or var2
@@ -681,26 +791,33 @@ lav_start_check_cov <- function(lavpartable = NULL, start = lavpartable$start) {
                 # we prefer a free parameter, and not user-specified
                 if(        lavpartable$free[this.cov.idx] > 0L &&
                    is.na(lavpartable$ustart[this.cov.idx])) {
+                    warning(txt)
                     start[this.cov.idx] <- start[this.cov.idx] / (COR * 1.1)
                 } else if( lavpartable$free[var.min.idx] > 0L &&
                            is.na(lavpartable$ustart[var.min.idx])) {
+                    warning(txt)
                     start[var.min.idx] <- start[var.min.idx] * (COR * 1.1)^2
                 } else if(        lavpartable$free[var.max.idx] > 0L &&
                           is.na(lavpartable$ustart[var.max.idx])) {
+                    warning(txt)
                     start[var.max.idx] <- start[var.max.idx] * (COR * 1.1)^2
 
                 # not found? try just a free parameter
                 } else if      (lavpartable$free[this.cov.idx] > 0L) {
+                    warning(txt)
                     start[this.cov.idx] <- start[this.cov.idx] / (COR * 1.1)
                 } else if(     lavpartable$free[var.min.idx] > 0L) {
+                    warning(txt)
                     start[var.min.idx] <- start[var.min.idx] * (COR * 1.1)^2
                 } else if(     lavpartable$free[var.max.idx] > 0L) {
+                    warning(txt)
                     start[var.max.idx] <- start[var.max.idx] * (COR * 1.1)^2
 
-                # nothing? abort
+                # nothing? abort or warn (and fail later...): warn
                 } else {
-                    stop("lavaan ERROR: please provide better fixed values for (co)variances;\n",
-"                variables involved are: ", var.lhs, " ", var.rhs, block.txt)
+                    warning(txt)
+                    #stop("lavaan ERROR: please provide better fixed values for (co)variances;\n",
+#"                variables involved are: ", var.lhs, " ", var.rhs, block.txt)
                 }
             } # COR > 1
         } # cov.idx
